<?php
/*
Plugin Name: DD Last Viewed
Version: 3.0.3
Plugin URI: http://dijkstradesign.com
Description: Shows the users recently viewed/visited Posts, Pages, Custom Types and even Terms in a widget.
Author: Wouter Dijkstra
Author URI: http://dijkstradesign.com
*/


/*  Copyright 2017  WOUTER DIJKSTRA  (email : info@dijkstradesign.nl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class lastViewed extends WP_Widget
{

    function __construct()
    {

        $widget_ops = array(
            'classname' => 'dd_last_viewed',
            'description' => __("A list of the recently viewed posts, pages or custom posttypes.")
        );
        parent::__construct('dd_last_viewed', _x('DD Last Viewed', 'DD Last Viewed widget'), $widget_ops);

        add_action('customize_preview_init', array($this, 'my_preview_js'));
        add_action('wp_enqueue_scripts', array($this, 'dd_lastviewed_add_front'));
        add_action('admin_init', array($this, 'dd_lastviewed_admin'));
        add_action('get_header', array($this, 'add_lastviewed_id'));
        add_shortcode('dd_lastviewed', array($this, 'shortcode_lastviewed'));
    }

    function my_preview_js()
    {
        wp_enqueue_script('dd_js_admin-lastviewed', plugins_url('/js/admin-lastviewed.js', __FILE__), array('jquery'), '');
    }

    function dd_lastviewed_add_front()
    {
        wp_register_style('dd_lastviewed_css', plugins_url('/css/style.css', __FILE__));
        wp_enqueue_style('dd_lastviewed_css');
    }

    function dd_lastviewed_admin()
    {
        wp_register_style('dd_lastviewed_admin_styles', plugins_url('/css/admin-style.css', __FILE__));
        wp_enqueue_style('dd_lastviewed_admin_styles');
        wp_enqueue_script('jquery');
        wp_enqueue_script('select-2', plugins_url('/js/select2.min.js', __FILE__), array('jquery'), '');
        wp_enqueue_script('dd_js_admin-lastviewed', plugins_url('/js/admin-lastviewed.js', __FILE__), array(
            'jquery',
            'select-2'
        ), '');
    }

    function add_lastviewed_id()
    {

        if (is_singular()) {

            global $post;
            $post_id = $post->ID;
            $post_type = get_post_type();
            $lastviewed_widgets = get_option('widget_dd_last_viewed');
            $post_selected_terms = $this->get_all_selected_terms($post_id);
            array_push($post_selected_terms, $post_type);

            foreach ($lastviewed_widgets as $id => $lastviewed_widget) {

                if ($id != '_multiwidget') :

                    $selection = $lastviewed_widget["selection"] ? $lastviewed_widget["selection"] : array();
                    $matching_selection = array_intersect($selection, $post_selected_terms);
                    $exclude_ids = explode(',', $lastviewed_widget["lastviewed_excl_ids"]);
                    $exclude_post = in_array($post_id, $exclude_ids); //true/false

                    if (!empty($matching_selection) && !$exclude_post) {

                        $cookieName = "cookie_data_lastviewed_widget_" . $id;
                        $cookieVal = $_COOKIE[$cookieName];
                        $oldList = explode(',', $cookieVal);
                        $newList = isset($cookieVal) ? array_diff($oldList, array($post_id)) : array();
                        array_push($newList, $post_id);
                        $newList = implode(",", $newList);
                        setcookie($cookieName, $newList, time() + (60 * 60 * 24 * 30), "/"); // 30 days
                    }
                endif;
            }
        }
    }

    function shortcode_lastviewed($atts)
    {

        $args = array(
            'widget_id' => $atts['widget_id'],
            'by_shortcode' => 'shortcode_',
        );

        ob_start();
        the_widget('lastviewed', '', $args);
        $output = ob_get_clean();

        return $output;
    }

    function get_all_selected_terms($post_id)
    {

        $selected_terms = array();
        $args = array('hide_empty' => 1, 'fields' => 'ids');
        $taxonomies = get_taxonomies();

        foreach ($taxonomies as $taxonomy) {
            $termID = wp_get_post_terms($post_id, $taxonomy, $args);
            $selected_terms = array_merge($selected_terms, $termID);
        }

        return $selected_terms;
    }

    function form($instance)
    {
        include('views/form.php');
    }

    function update($new_instance, $old_instance)
    {
        // processes widget options to be saved
        $instance = $old_instance;
        $instance['lastviewedTitle'] = strip_tags($new_instance['lastviewedTitle']);
        $instance['selected_posttypes'] = $new_instance['selected_posttypes'];
        $instance['selection'] = $new_instance['selection'];

        $instance['lastViewed_thumb'] = strip_tags($new_instance['lastViewed_thumb']);
        $instance['lastViewed_total'] = strip_tags($new_instance['lastViewed_total']);
        $instance['lastViewed_truncate'] = strip_tags($new_instance['lastViewed_truncate']);
        $instance['lastViewed_linkname'] = strip_tags($new_instance['lastViewed_linkname']);
        $instance['lastViewed_showPostTitle'] = (bool)$new_instance['lastViewed_showPostTitle'];
        $instance['lastViewed_showThumb'] = (bool)$new_instance['lastViewed_showThumb'];
        $instance['lastViewed_thumbSize'] = strip_tags($new_instance['lastViewed_thumbSize']);
        $instance['lastViewed_showExcerpt'] = (bool)$new_instance['lastViewed_showExcerpt'];
        $instance['lastViewed_content_type'] = strip_tags($new_instance['lastViewed_content_type']);
        $instance['lastViewed_showTruncate'] = (bool)$new_instance['lastViewed_showTruncate'];
        $instance['lastViewed_showMore'] = (bool)$new_instance['lastViewed_showMore'];
        $instance['lastViewed_lv_link_thumb'] = (bool)$new_instance['lastViewed_lv_link_thumb'];
        $instance['lastViewed_lv_link_title'] = (bool)$new_instance['lastViewed_lv_link_title'];
        $instance['lastViewed_lv_link_excerpt'] = (bool)$new_instance['lastViewed_lv_link_excerpt'];
        $instance['lastviewed_excl_ids'] = strip_tags($new_instance['lastviewed_excl_ids']);

        return $instance;
    }

    function is_posttype_checked($post_type, $selected_post_types = array())
    {
        $selected_post_types = array_values($selected_post_types);

        return in_array($post_type, $selected_post_types);
    }

    function widget($args, $instance)
    {
        include('views/widget.php');
    }
}

add_action('widgets_init', create_function('', 'return register_widget("lastviewed");'));